/**
 * Stuff that all nodes and branches in the
 * sound tree know.
 **/
abstract public class CollectableNode {
  /**
   * The next branch/node/whatever to process
   **/
  public CollectableNode next;
  
  /**
   * Constructor for CollectableNode just sets
   * next to null
   **/
  public CollectableNode(){
    next = null;
  }
  
  /**
   * Methods to set and get next elements
   * @param nextOne next element in list
   **/
  public void setNext(CollectableNode nextOne){
    this.next = nextOne;
  }
  
  public CollectableNode getNext(){
    return this.next;
  }
  
  /**
   * Play the list of sound elements
   * after me
   **/
  public void playFromMeOn(){
    this.collect().play();
  }
  
  /** 
   * Collect all the sounds from me on
   **/
  abstract public Sound collect();
  
  /** 
   * Collect all the sounds from me on,
   * but if there's processing, do it after.
   **/
  abstract public Sound collectAfter();

  /** Method to remove node from list, fixing links appropriately.
   * @param node element to remove from list.
   **/
  public void remove(CollectableNode node){
    if (node==this)
    {
      System.out.println("I can't remove the first node from the list.");
      return;
    };
    
    CollectableNode current = this;
    // While there are more nodes to consider
    while (current.getNext() != null)
    {
      if (current.getNext() == node){
        // Simply make node's next be this next
        current.setNext(node.getNext());
        // Make this node point to nothing
        node.setNext(null);
        return;
      }
      current = current.getNext();
    }
  }

    
  /**
   * Insert the input node after this node.
   * @param node element to insert after this.
   **/
  public void insertAfter(CollectableNode node){
    // Save what "this" currently points at
    CollectableNode oldNext = this.getNext();
    this.setNext(node);
    node.setNext(oldNext);
  }
  
  /**
   * Return the last element in the list
   **/
    public CollectableNode last() {
      CollectableNode current;
    
      current = this;
      while (current.getNext() != null)
      {
        current = current.getNext();
      };
      return current;
  }

  /**
   * Add the input node after the last node in this list.
   * @param node element to insert after this.
   **/
  public void add(CollectableNode node){
    this.last().insertAfter(node);
  }

}
    
